###############################################################################
#                                                                             #
#                             Get-RemovedMailbox                              #
#                                    V2.90                                    #
#                       (C)2006-2014 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)]: $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_errors_count
{
Param($UserID)
$S="ErrorsCount[$($UserID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function welcome
{
write-host "Get Removed Mailbox for Office 365 $($Version)" -foregroundcolor "white"
write-host "(C) 2006-2014 KoXo Developpement" -foregroundcolor "white"
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword,$ProxyEnabled,$ProxyUser,$ProxyPassword)
$error.Clear()
#Proxy
if ($ProxyEnabled)
   {
   if (![String]::IsNullorEmpty($ProxyUser))
      { 
      $SecuredPassword=ConvertTo-SecureString $ProxyPassword -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $ProxyUser, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection with Internet Explorer proxy parameters, User=$($ProxyUser)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection with Internet Explorer proxy parameters"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
if ($ProxyEnabled)
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -SessionOption $PSSessionOption -ErrorAction SilentlyContinue
   }
   else
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
   }
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
 return $Cred
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($Error[0]))
	{
  $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
  $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
  log_message "ERROR" $S
	}
}
###############################################################################
function removedmailbox
{
$Error.Clear()
$User=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue {Get-RemovedMailbox}
return $User
}
###############################################################################
function exec
{
$Log.Text=$Log.Text+"------ Get-RemovedMailbox ------"
$Info = removedmailbox | Out-String
$Log.Text=$Log.Text+$Info
}
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V2.90"
$global:total_errors_count=0
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword="%OFFICE365_PASSWORD%"
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword="%PROXY_PASSWORD%"
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
welcome
check_version
connect $Office365Url $Office365LoginName $Office365LoginPassword $ProxyEnabled $ProxyUser $ProxyPassword
if ($global:total_errors_count -eq 0)
    {
    [void][Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
    [void][Reflection.Assembly]::LoadWithPartialName("System.Drawing")
    $panel1 = new-object System.Windows.Forms.Panel
    $buttonClose = new-object System.Windows.Forms.Button
    $groupBox1 = new-object System.Windows.Forms.GroupBox
    $Log = new-object System.Windows.Forms.TextBox
    #
    # panel1
    #
    $panel1.Anchor = [System.Windows.Forms.AnchorStyles]"Bottom,Left,Right"
    $panel1.BackColor = [System.Drawing.Color]::FromArgb(30,139,225)
    $panel1.Controls.Add($buttonClose)
    $panel1.Location = new-object System.Drawing.Point(0, 402)
    $panel1.Name = "panel1"
    $panel1.Size = new-object System.Drawing.Size(673, 35)
    $panel1.TabIndex = 0
    #
    # buttonClose
    #
    $buttonClose.Anchor = [System.Windows.Forms.AnchorStyles]"Top,Bottom,Right"
    $buttonClose.Location = new-object System.Drawing.Point(579, 5)
    $buttonClose.Name = "buttonClose"
    $buttonClose.Size = new-object System.Drawing.Size(89, 25)
    $buttonClose.TabIndex = 0
    $buttonClose.Text = "&Close"
    $buttonClose.UseVisualStyleBackColor = $true
    function OnClick_buttonClose($Sender,$e){$MyForm.Close()}
    $buttonClose.Add_Click( { OnClick_buttonClose $buttonClose $EventArgs} )
    #
    # groupBox1
    #
    $groupBox1.Anchor = [System.Windows.Forms.AnchorStyles]"Top,Bottom,Left,Right"
    $groupBox1.BackColor = [System.Drawing.Color]::FromArgb(30,139,225)
    $groupBox1.Controls.Add($Log)
    $groupBox1.Location = new-object System.Drawing.Point(0, 0)
    $groupBox1.Name = "groupBox1"
    $groupBox1.Size = new-object System.Drawing.Size(672, 402)
    $groupBox1.TabIndex = 2
    $groupBox1.TabStop = $false
    $groupBox1.Text = " Removed Mailbox "
    #
    # Log
    #
    $Log.AcceptsReturn = $true
    $Log.AcceptsTab = $true
    $Log.Anchor = [System.Windows.Forms.AnchorStyles]"Top,Bottom,Left,Right"
    $Log.BackColor = [System.Drawing.Color]::FromArgb(151,192,234)
    $Log.Cursor =[System.Windows.Forms.Cursors]::Default
    $Log.Font = new-object System.Drawing.Font("Courier New",10,[System.Drawing.FontStyle]::Bold)
    $Log.Location = new-object System.Drawing.Point(6, 16)
    $Log.Multiline = $true
    $Log.Name = "Log"
    $Log.ScrollBars =[System.Windows.Forms.ScrollBars]::Both
    $Log.Size = new-object System.Drawing.Size(660, 382)
    $Log.TabIndex = 0
    $Log.WordWrap = $false
    #
    $MyForm = new-object System.Windows.Forms.form
    #
    $MyForm.AutoSize = $true
    $MyForm.ClientSize = new-object System.Drawing.Size(672, 440)
    $MyForm.Controls.Add($groupBox1)
    $MyForm.Controls.Add($panel1)
    $MyForm.Name = "MyForm"
    $MyForm.StartPosition =[System.Windows.Forms.FormStartPosition]::CenterScreen
    $MyForm.Text = "(C)2006-2014 KoXo Developpement | Office 365 Powershell Log"
    #
    exec 
    #
    $MyForm.Topmost = $False
    $MyForm.Add_Shown({$MyForm.Activate()})
    [void] $MyForm.ShowDialog()
    $MyForm.Dispose()
    # close
    }
log_total_errors_count
